@@ -2,12 +2,14 @@ import sys
 import re
 import os
 import datetime
+import uuid
 import logging
 import warnings
+import string
 import time
 from ast import literal_eval
 from decimal import Decimal, localcontext
-from collections import namedtuple, OrderedDict
+from collections import namedtuple
 from itertools import repeat
 from ordered_set import OrderedSet
 from threading import Timer
@@ -40,6 +42,7 @@ except ImportError:  # pragma: no cover. The case without Numpy is tested locall
     np_complex64 = np_type  # pragma: no cover.
     np_complex128 = np_type  # pragma: no cover.
     np_complex_ = np_type  # pragma: no cover.
+    np_complexfloating = np_type  # pragma: no cover.
 else:
     np_array_factory = np.array
     np_ndarray = np.ndarray
@@ -60,6 +63,7 @@ else:
     np_complex64 = np.complex64
     np_complex128 = np.complex128
     np_complex_ = np.complex_
+    np_complexfloating = np.complexfloating
 
 numpy_numbers = (
     np_int8, np_int16, np_int32, np_int64, np_uint8,
@@ -67,6 +71,10 @@ numpy_numbers = (
     np_float32, np_float64, np_float_, np_complex64,
     np_complex128, np_complex_,)
 
+numpy_complex_numbers = (
+    np_complexfloating, np_complex64, np_complex128, np_complex_,
+)
+
 numpy_dtypes = set(numpy_numbers)
 numpy_dtypes.add(np_bool_)
 
@@ -85,32 +93,11 @@ py2 = py_major_version == 2
 py3 = py_major_version == 3
 py4 = py_major_version == 4
 
-MINIMUM_PY_DICT_TYPE_SORTED = Decimal('3.6')
-DICT_IS_SORTED = py_current_version >= MINIMUM_PY_DICT_TYPE_SORTED
 
+NUMERICS = frozenset(string.digits)
 
-class OrderedDictPlus(OrderedDict):
-    """
-    This class is only used when a python version is used where
-    the built-in dictionary is not ordered.
-    """
-
-    def __repr__(self):  # pragma: no cover. Only used in pypy3 and py3.5
-        return str(dict(self))  # pragma: no cover. Only used in pypy3 and py3.5
-
-    __str__ = __repr__
-
-    def copy(self):  # pragma: no cover. Only used in pypy3 and py3.5
-        result = OrderedDictPlus()  # pragma: no cover. Only used in pypy3 and py3.5
-        for k, v in self.items():  # pragma: no cover. Only used in pypy3 and py3.5
-            result[k] = v  # pragma: no cover. Only used in pypy3 and py3.5
-        return result  # pragma: no cover. Only used in pypy3 and py3.5
-
-
-if DICT_IS_SORTED:
-    dict_ = dict
-else:
-    dict_ = OrderedDictPlus  # pragma: no cover. Only used in pypy3 and py3.5
+# we used to use OrderedDictPlus when dictionaries in Python were not ordered.
+dict_ = dict
 
 if py4:
     logger.warning('Python 4 is not supported yet. Switching logic to Python 3.')  # pragma: no cover
@@ -124,26 +111,30 @@ pypy3 = py3 and hasattr(sys, "pypy_translation_info")
 strings = (str, bytes)  # which are both basestring
 unicode_type = str
 bytes_type = bytes
+only_complex_number = (complex,) + numpy_complex_numbers
 only_numbers = (int, float, complex, Decimal) + numpy_numbers
 datetimes = (datetime.datetime, datetime.date, datetime.timedelta, datetime.time)
+uuids = (uuid.UUID, )
 times = (datetime.datetime, datetime.time)
 numbers = only_numbers + datetimes
 booleans = (bool, np_bool_)
 
+basic_types = strings + numbers + uuids + booleans + (type(None), )
+
 IndexedHash = namedtuple('IndexedHash', 'indexes item')
 
 current_dir = os.path.dirname(os.path.abspath(__file__))
 
 ID_PREFIX = '!>*id'
 
-ZERO_DECIMAL_CHARACTERS = set("-0.")
-
 KEY_TO_VAL_STR = "{}:{}"
 
 TREE_VIEW = 'tree'
 TEXT_VIEW = 'text'
 DELTA_VIEW = '_delta'
 
+ENUM_INCLUDE_KEYS = ['__objclass__', 'name', 'value']
+
 
 def short_repr(item, max_length=15):
     """Short representation of item if it is too long"""
@@ -239,28 +230,6 @@ class indexed_set(set):
     """
 
 
-JSON_CONVERTOR = {
-    Decimal: float,
-    OrderedSet: list,
-    type: lambda x: x.__name__,
-    bytes: lambda x: x.decode('utf-8')
-}
-
-
-def json_convertor_default(default_mapping=None):
-    _convertor_mapping = JSON_CONVERTOR.copy()
-    if default_mapping:
-        _convertor_mapping.update(default_mapping)
-
-    def _convertor(obj):
-        for original_type, convert_to in _convertor_mapping.items():
-            if isinstance(obj, original_type):
-                return convert_to(obj)
-        raise TypeError('We do not know how to convert {} of type {} for json serialization. Please pass the default_mapping parameter with proper mapping of the object to a basic python type.'.format(obj, type(obj)))
-
-    return _convertor
-
-
 def add_to_frozen_set(parents_ids, item_id):
     return parents_ids | {item_id}
 
@@ -276,6 +245,31 @@ def convert_item_or_items_into_set_else_none(items):
     return items
 
 
+def add_root_to_paths(paths):
+    """
+    Sometimes the users want to just pass
+    [key] instead of root[key] for example.
+    Here we automatically add all sorts of variations that might match
+    the path they were supposed to pass. 
+    """
+    if paths is None:
+        return
+    result = OrderedSet()
+    for path in paths:
+        if path.startswith('root'):
+            result.add(path)
+        else:
+            if path.isdigit():
+                result.add(f"root['{path}']")
+                result.add(f"root[{path}]")
+            elif path[0].isdigit():
+                result.add(f"root['{path}']")
+            else:
+                result.add(f"root.{path}")
+                result.add(f"root['{path}']")
+    return result
+
+
 RE_COMPILED_TYPE = type(re.compile(''))
 
 
@@ -342,20 +336,51 @@ def number_to_string(number, significant_digits, number_format_notation="f"):
         using = number_formatting[number_format_notation]
     except KeyError:
         raise ValueError("number_format_notation got invalid value of {}. The valid values are 'f' and 'e'".format(number_format_notation)) from None
-    if isinstance(number, Decimal):
-        tup = number.as_tuple()
-        with localcontext() as ctx:
-            ctx.prec = len(tup.digits) + tup.exponent + significant_digits
-            number = number.quantize(Decimal('0.' + '0' * significant_digits))
-    elif not isinstance(number, numbers):
+    
+    if not isinstance(number, numbers):
         return number
+    elif isinstance(number, Decimal):
+        with localcontext() as ctx:
+            # Precision = number of integer digits + significant_digits
+            # Using number//1 to get the integer part of the number
+            ctx.prec = len(str(abs(number // 1))) + significant_digits
+            number = number.quantize(Decimal('0.' + '0' * significant_digits))
+    elif isinstance(number, only_complex_number):
+        # Case for complex numbers.
+        number = number.__class__(
+            "{real}+{imag}j".format(
+                real=number_to_string(
+                    number=number.real,
+                    significant_digits=significant_digits,
+                    number_format_notation=number_format_notation
+                ),
+                imag=number_to_string(
+                    number=number.imag,
+                    significant_digits=significant_digits,
+                    number_format_notation=number_format_notation
+                )
+            )
+        )
+    else:
+        number = round(number=number, ndigits=significant_digits)
+
+        if significant_digits == 0:
+            number = int(number)
+
+    if number == 0.0:
+        # Special case for 0: "-0.xx" should compare equal to "0.xx"
+        number = abs(number)
+
+    # Cast number to string
     result = (using % significant_digits).format(number)
-    # Special case for 0: "-0.00" should compare equal to "0.00"
-    if set(result) <= ZERO_DECIMAL_CHARACTERS:
-        result = "0.00"
     # https://bugs.python.org/issue36622
-    if number_format_notation == 'e' and isinstance(number, float):
-        result = result.replace('+0', '+')
+    if number_format_notation == 'e':
+        # Removing leading 0 for exponential part.
+        result = re.sub(
+            pattern=r'(?<=e(\+|\-))0(?=\d)+',
+            repl=r'',
+            string=result
+        )
     return result
 
 
@@ -516,7 +541,7 @@ LITERAL_EVAL_PRE_PROCESS = [
 
 def literal_eval_extended(item):
     """
-    An extend version of literal_eval
+    An extended version of literal_eval
     """
     try:
         return literal_eval(item)
@@ -603,3 +628,39 @@ def get_homogeneous_numpy_compatible_type_of_seq(seq):
         return PYTHON_TYPE_TO_NUMPY_TYPE.get(type_, False)
     else:
         return False
+
+
+def detailed__dict__(obj, ignore_private_variables=True, ignore_keys=frozenset(), include_keys=None):
+    """
+    Get the detailed dictionary of an object.
+
+    This is used so we retrieve object properties too.
+    """
+    if include_keys:
+        result = {}
+        for key in include_keys:
+            try:
+                value = getattr(obj, key)
+            except Exception:
+                pass
+            else:
+                if not callable(value) or key == '__objclass__':  # We don't want to compare functions, however for backward compatibility, __objclass__ needs to be reported.
+                    result[key] = value
+    else:
+        result = obj.__dict__.copy()  # A shallow copy
+        private_var_prefix = f"_{obj.__class__.__name__}__"  # The semi private variables in Python get this prefix
+        for key in ignore_keys:
+            if key in result or (
+                ignore_private_variables and key.startswith('__') and not key.startswith(private_var_prefix)
+            ):
+                del result[key]
+        for key in dir(obj):
+            if key not in result and key not in ignore_keys and (
+                    not ignore_private_variables or (
+                        ignore_private_variables and not key.startswith('__') and not key.startswith(private_var_prefix)
+                    )
+            ):
+                value = getattr(obj, key)
+                if not callable(value):
+                    result[key] = value
+    return result
