@@ -1,8 +1,9 @@
-import json
 import pickle
 import sys
 import io
 import os
+import json
+import uuid
 import logging
 import re  # NOQA
 import builtins  # NOQA
@@ -20,11 +21,14 @@ except ImportError:  # pragma: no cover.
     toml = None  # pragma: no cover.
 try:
     import clevercsv
+    csv = None
 except ImportError:  # pragma: no cover.
+    import csv
     clevercsv = None  # pragma: no cover.
 from copy import deepcopy
+from functools import partial
 from collections.abc import Mapping
-from deepdiff.helper import (strings, json_convertor_default, get_type, TEXT_VIEW)
+from deepdiff.helper import (strings, get_type, TEXT_VIEW)
 from deepdiff.model import DeltaResult
 
 logger = logging.getLogger(__name__)
@@ -69,14 +73,42 @@ SAFE_TO_IMPORT = {
     'datetime.time',
     'datetime.timedelta',
     'decimal.Decimal',
+    'uuid.UUID',
     'ordered_set.OrderedSet',
     'collections.namedtuple',
     'collections.OrderedDict',
-    'deepdiff.helper.OrderedDictPlus',
     're.Pattern',
 }
 
 
+TYPE_STR_TO_TYPE = {
+    'range': range,
+    'complex': complex,
+    'set': set,
+    'frozenset': frozenset,
+    'slice': slice,
+    'str': str,
+    'bytes': bytes,
+    'list': list,
+    'tuple': tuple,
+    'int': int,
+    'float': float,
+    'dict': dict,
+    'bool': bool,
+    'bin': bin,
+    'None': None,
+    'NoneType': None,
+    'datetime': datetime.datetime,
+    'time': datetime.time,
+    'timedelta': datetime.timedelta,
+    'Decimal': decimal.Decimal,
+    'OrderedSet': ordered_set.OrderedSet,
+    'namedtuple': collections.namedtuple,
+    'OrderedDict': collections.OrderedDict,
+    'Pattern': re.Pattern,    
+}
+
+
 class ModuleNotFoundError(ImportError):
     """
     Raised when the module is not found in sys.modules
@@ -326,19 +358,32 @@ def pickle_load(content, safe_to_import=None):
     return _RestrictedUnpickler(io.BytesIO(content), safe_to_import=safe_to_import).load()
 
 
-PRETTY_FORM_TEXTS = {
-    "type_changes": "Type of {diff_path} changed from {type_t1} to {type_t2} and value changed from {val_t1} to {val_t2}.",
-    "values_changed": "Value of {diff_path} changed from {val_t1} to {val_t2}.",
-    "dictionary_item_added": "Item {diff_path} added to dictionary.",
-    "dictionary_item_removed": "Item {diff_path} removed from dictionary.",
-    "iterable_item_added": "Item {diff_path} added to iterable.",
-    "iterable_item_removed": "Item {diff_path} removed from iterable.",
-    "attribute_added": "Attribute {diff_path} added.",
-    "attribute_removed": "Attribute {diff_path} removed.",
-    "set_item_added": "Item root[{val_t2}] added to set.",
-    "set_item_removed": "Item root[{val_t1}] removed from set.",
-    "repetition_change": "Repetition change for item {diff_path}.",
-}
+def _get_pretty_form_text(verbose_level):
+    pretty_form_texts = {
+        "type_changes": "Type of {diff_path} changed from {type_t1} to {type_t2} and value changed from {val_t1} to {val_t2}.",
+        "values_changed": "Value of {diff_path} changed from {val_t1} to {val_t2}.",
+        "dictionary_item_added": "Item {diff_path} added to dictionary.",
+        "dictionary_item_removed": "Item {diff_path} removed from dictionary.",
+        "iterable_item_added": "Item {diff_path} added to iterable.",
+        "iterable_item_removed": "Item {diff_path} removed from iterable.",
+        "attribute_added": "Attribute {diff_path} added.",
+        "attribute_removed": "Attribute {diff_path} removed.",
+        "set_item_added": "Item root[{val_t2}] added to set.",
+        "set_item_removed": "Item root[{val_t1}] removed from set.",
+        "repetition_change": "Repetition change for item {diff_path}.",
+    }
+    if verbose_level == 2:
+        pretty_form_texts.update(
+            {
+                "dictionary_item_added": "Item {diff_path} ({val_t2}) added to dictionary.",
+                "dictionary_item_removed": "Item {diff_path} ({val_t1}) removed from dictionary.",
+                "iterable_item_added": "Item {diff_path} ({val_t2}) added to iterable.",
+                "iterable_item_removed": "Item {diff_path} ({val_t1}) removed from iterable.",
+                "attribute_added": "Attribute {diff_path} ({val_t2}) added.",
+                "attribute_removed": "Attribute {diff_path} ({val_t1}) removed.",
+            }
+        )
+    return pretty_form_texts
 
 
 def pretty_print_diff(diff):
@@ -349,7 +394,7 @@ def pretty_print_diff(diff):
     val_t2 = '"{}"'.format(str(diff.t2)) if type_t2 == "str" else str(diff.t2)
 
     diff_path = diff.path(root='root')
-    return PRETTY_FORM_TEXTS.get(diff.report_type, "").format(
+    return _get_pretty_form_text(diff.verbose_level).get(diff.report_type, "").format(
         diff_path=diff_path,
         type_t1=type_t1,
         type_t2=type_t2,
@@ -381,9 +426,11 @@ def load_path_content(path, file_type=None):
             content = the_file.read()
             content = pickle_load(content)
     elif file_type in {'csv', 'tsv'}:
-        if clevercsv is None:  # pragma: no cover.
-            raise ImportError('CleverCSV needs to be installed.')  # pragma: no cover.
-        content = clevercsv.read_dicts(path)
+        if clevercsv:  # pragma: no cover.
+            content = clevercsv.read_dicts(path)
+        else:
+            with open(path, 'r') as the_file:
+                content = list(csv.DictReader(the_file))
         logger.info(f"NOTE: CSV content was empty in {path}")
 
         # Everything in csv is string but we try to automatically convert any numbers we find
@@ -442,14 +489,70 @@ def _save_content(content, path, file_type, keep_backup=True):
         with open(path, 'wb') as the_file:
             content = pickle_dump(content, file_obj=the_file)
     elif file_type in {'csv', 'tsv'}:
-        if clevercsv is None:  # pragma: no cover.
-            raise ImportError('CleverCSV needs to be installed.')  # pragma: no cover.
+        if clevercsv:  # pragma: no cover.
+            dict_writer = clevercsv.DictWriter
+        else:
+            dict_writer = csv.DictWriter
         with open(path, 'w', newline='') as csvfile:
             fieldnames = list(content[0].keys())
-            writer = clevercsv.DictWriter(csvfile, fieldnames=fieldnames)
+            writer = dict_writer(csvfile, fieldnames=fieldnames)
             writer.writeheader()
             writer.writerows(content)
     else:
         raise UnsupportedFormatErr('Only json, yaml, toml, csv, tsv and pickle are supported.\n'
                                    f' The {file_type} extension is not known.')
     return content
+
+
+JSON_CONVERTOR = {
+    decimal.Decimal: float,
+    ordered_set.OrderedSet: list,
+    type: lambda x: x.__name__,
+    bytes: lambda x: x.decode('utf-8'),
+    datetime.datetime: lambda x: x.isoformat(),
+    uuid.UUID: lambda x: str(x),
+}
+
+
+def json_convertor_default(default_mapping=None):
+    if default_mapping:
+        _convertor_mapping = JSON_CONVERTOR.copy()
+        _convertor_mapping.update(default_mapping)
+    else:
+        _convertor_mapping = JSON_CONVERTOR
+
+    def _convertor(obj):
+        for original_type, convert_to in _convertor_mapping.items():
+            if isinstance(obj, original_type):
+                return convert_to(obj)
+        raise TypeError('We do not know how to convert {} of type {} for json serialization. Please pass the default_mapping parameter with proper mapping of the object to a basic python type.'.format(obj, type(obj)))
+
+    return _convertor
+
+
+class JSONDecoder(json.JSONDecoder):
+
+    def __init__(self, *args, **kwargs):
+        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)
+
+    def object_hook(self, obj):
+        if 'old_type' in obj and 'new_type' in obj:
+            for type_key in ('old_type', 'new_type'):
+                type_str = obj[type_key]
+                obj[type_key] = TYPE_STR_TO_TYPE.get(type_str, type_str)
+
+        return obj
+
+
+def json_dumps(item, default_mapping=None, **kwargs):
+    """
+    Dump json with extra details that are not normally json serializable
+
+    Note: I tried to replace json with orjson for its speed. It does work
+    but the output it makes is a byte object and Postgres couldn't directly use it without
+    encoding to str. So I switched back to json.
+    """
+    return json.dumps(item, default=json_convertor_default(default_mapping=default_mapping), **kwargs)
+
+
+json_loads = partial(json.loads, cls=JSONDecoder)
