@@ -16,7 +16,8 @@ from deepdiff.delta import (
     INVALID_ACTION_WHEN_CALLING_SIMPLE_DELETE_ELEM, INDEXES_NOT_FOUND_WHEN_IGNORE_ORDER,
     FAIL_TO_REMOVE_ITEM_IGNORE_ORDER_MSG, UNABLE_TO_GET_PATH_MSG, NOT_VALID_NUMPY_TYPE)
 from deepdiff.serialization import (
-    DELTA_IGNORE_ORDER_NEEDS_REPETITION_REPORT, DELTA_ERROR_WHEN_GROUP_BY
+    DELTA_IGNORE_ORDER_NEEDS_REPETITION_REPORT, DELTA_ERROR_WHEN_GROUP_BY,
+    json_dumps, json_loads,
 )
 
 from tests import PicklableClass, parameterize_cases, CustomClass, CustomClass2
@@ -24,6 +25,35 @@ from tests import PicklableClass, parameterize_cases, CustomClass, CustomClass2
 
 class TestBasicsOfDelta:
 
+    def test_from_null_delta_json(self):
+        t1 = None
+        t2 = [1, 2, 3, 5]
+        diff = DeepDiff(t1, t2)
+        delta = Delta(diff, serializer=json_dumps)
+        dump = delta.dumps()
+        delta2 = Delta(dump, deserializer=json_loads)
+        assert delta2 + t1 == t2
+        assert t1 + delta2 == t2
+
+    def test_to_null_delta1_json(self):
+        t1 = 1
+        t2 = None
+        diff = DeepDiff(t1, t2)
+        delta = Delta(diff, serializer=json_dumps)
+        dump = delta.dumps()
+        delta2 = Delta(dump, deserializer=json_loads)
+        assert delta2 + t1 == t2
+        assert t1 + delta2 == t2
+
+    def test_to_null_delta2_json(self):
+        t1 = [1, 2, 3, 5]
+        t2 = None
+        diff = DeepDiff(t1, t2)
+        delta = Delta(diff)
+
+        assert delta + t1 == t2
+        assert t1 + delta == t2
+
     def test_list_difference_add_delta(self):
         t1 = [1, 2]
         t2 = [1, 2, 3, 5]
@@ -604,21 +634,28 @@ DELTA_CASES = {
         'to_delta_kwargs': {},
         'expected_delta_dict': {'iterable_item_removed': {'root[9]': 'a', 'root[10]': 'b', 'root[11]': 'c'}}
     },
+    'delta_case19_value_removed_from_the_middle_of_list': {
+        't1': [0, 1, 2, 3, 4, 5, 6, 7, 8, 'a', 'b', 'c'],
+        't2': [0, 1, 2, 3, 5, 6, 7, 8, 'a', 'b', 'c'],
+        'deepdiff_kwargs': {},
+        'to_delta_kwargs': {'directed': True},
+        'expected_delta_dict': {'iterable_item_removed': {'root[4]': 4}}
+    },
 }
 
 
-DELTA_CASES_PARAMS = parameterize_cases('t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict', DELTA_CASES)
+DELTA_CASES_PARAMS = parameterize_cases('test_name, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict', DELTA_CASES)
 
 
 class TestDelta:
 
     @pytest.mark.parametrize(**DELTA_CASES_PARAMS)
-    def test_delta_cases(self, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict):
+    def test_delta_cases(self, test_name, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict):
         diff = DeepDiff(t1, t2, **deepdiff_kwargs)
         delta_dict = diff._to_delta_dict(**to_delta_kwargs)
-        assert expected_delta_dict == delta_dict
+        assert expected_delta_dict == delta_dict, f"test_delta_cases {test_name} failed."
         delta = Delta(diff, verify_symmetry=False, raise_errors=True)
-        assert t1 + delta == t2
+        assert t1 + delta == t2, f"test_delta_cases {test_name} failed."
 
 
 DELTA_IGNORE_ORDER_CASES = {
@@ -901,15 +938,15 @@ DELTA_IGNORE_ORDER_CASES = {
 }
 
 DELTA_IGNORE_ORDER_CASES_PARAMS = parameterize_cases(
-    't1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict, expected_t1_plus_delta', DELTA_IGNORE_ORDER_CASES)
+    'test_name, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict, expected_t1_plus_delta', DELTA_IGNORE_ORDER_CASES)
 
 
 class TestIgnoreOrderDelta:
 
     @pytest.mark.parametrize(**DELTA_IGNORE_ORDER_CASES_PARAMS)
     def test_ignore_order_delta_cases(
-            self, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict, expected_t1_plus_delta, request):
-        test_name = request.node.callspec.id
+            self, test_name, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict, expected_t1_plus_delta, request):
+        # test_name = request.node.callspec.id
         diff = DeepDiff(t1, t2, **deepdiff_kwargs)
         delta_dict = diff._to_delta_dict(**to_delta_kwargs)
         assert expected_delta_dict == delta_dict, f"test_ignore_order_delta_cases {test_name} failed"
@@ -1064,31 +1101,31 @@ DELTA_NUMPY_TEST_CASES = {
 
 
 DELTA_NUMPY_TEST_PARAMS = parameterize_cases(
-    't1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict, expected_result', DELTA_NUMPY_TEST_CASES)
+    'test_name, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict, expected_result', DELTA_NUMPY_TEST_CASES)
 
 
 class TestNumpyDelta:
 
     @pytest.mark.parametrize(**DELTA_NUMPY_TEST_PARAMS)
-    def test_numpy_delta_cases(self, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict, expected_result):
+    def test_numpy_delta_cases(self, test_name, t1, t2, deepdiff_kwargs, to_delta_kwargs, expected_delta_dict, expected_result):
         diff = DeepDiff(t1, t2, **deepdiff_kwargs)
         delta_dict = diff._to_delta_dict(**to_delta_kwargs)
         if expected_delta_dict:
-            assert expected_delta_dict == delta_dict
+            assert expected_delta_dict == delta_dict, f"test_numpy_delta_cases {test_name} failed."
         delta = Delta(diff, verify_symmetry=False, raise_errors=True)
         if expected_result == 't2':
             result = delta + t1
-            assert np.array_equal(result, t2)
+            assert np.array_equal(result, t2), f"test_numpy_delta_cases {test_name} failed."
         elif expected_result == 't2_via_deepdiff':
             result = delta + t1
             diff = DeepDiff(result, t2, ignore_order=True, report_repetition=True)
-            assert not diff
+            assert not diff, f"test_numpy_delta_cases {test_name} failed."
         elif expected_result is DeltaNumpyOperatorOverrideError:
             with pytest.raises(DeltaNumpyOperatorOverrideError):
-                assert t1 + delta
+                t1 + delta
         else:
             result = delta + t1
-            assert np.array_equal(result, expected_result)
+            assert np.array_equal(result, expected_result), f"test_numpy_delta_cases {test_name} failed."
 
     def test_invalid_numpy_type(self):
 
@@ -1145,12 +1182,14 @@ class TestDeltaOther:
             'ignore_type_in_groups': [],
             'report_repetition': True,
             'exclude_paths': None,
+            'include_paths': None,
             'exclude_regex_paths': None,
             'exclude_types': None,
             'exclude_types_tuple': None,
             'ignore_type_subclasses': False,
             'ignore_string_case': False,
             'exclude_obj_callback': None,
+            'exclude_obj_callback_strict': None,
             'ignore_private_variables': True,
             'ignore_nan_inequality': False,
             'hasher': None,
@@ -1478,11 +1517,27 @@ class TestDeltaCompareFunc:
         t2 = [{'id': 3, 'val': 3}, {'id': 2, 'val': 2}, {'id': 1, 'val': 3}]
         ddiff = DeepDiff(t1, t2, iterable_compare_func=self.compare_func, verbose_level=2)
         expected = {
-            'values_changed': {"root[0]['val']": {'new_value': 3, 'old_value': 1}},
-            'iterable_item_removed': {'root[2]': {'id': 1, 'val': 3}},
-            'iterable_item_moved': {
-                'root[0]': {'new_path': 'root[2]', 'value': {'id': 1, 'val': 3}},
-                'root[3]': {'new_path': 'root[0]', 'value': {'id': 3, 'val': 3}}
+            "iterable_item_removed": {
+                "root[2]": {
+                    "id": 1,
+                    "val": 3
+                }
+            },
+            "iterable_item_moved": {
+                "root[0]": {
+                    "new_path": "root[2]",
+                    "value": {
+                        "id": 1,
+                        "val": 3
+                    }
+                },
+                "root[3]": {
+                    "new_path": "root[0]",
+                    "value": {
+                        "id": 3,
+                        "val": 3
+                    }
+                }
             }
         }
         assert expected == ddiff
@@ -1495,11 +1550,27 @@ class TestDeltaCompareFunc:
         t2 = [{'id': 1, 'val': 1}, {'id': 2, 'val': 2}, {'id': 1, 'val': 3}, {'id': 3, 'val': 3}]
         ddiff = DeepDiff(t1, t2, iterable_compare_func=self.compare_func, verbose_level=2)
         expected = {
-            'values_changed': {"root[2]['val']": {'new_value': 1, 'old_value': 3}},
-            'iterable_item_added': {'root[2]': {'id': 1, 'val': 3}},
+            'iterable_item_added': {
+                'root[2]': {
+                    'id': 1,
+                    'val': 3
+                }
+            },
             'iterable_item_moved': {
-                'root[2]': {'new_path': 'root[0]', 'value': {'id': 1, 'val': 1}},
-                'root[0]': {'new_path': 'root[3]', 'value': {'id': 3, 'val': 3}}
+                'root[0]': {
+                    'new_path': 'root[3]',
+                    'value': {
+                        'id': 3,
+                        'val': 3
+                    }
+                },
+                'root[2]': {
+                    'new_path': 'root[0]',
+                    'value': {
+                        'id': 1,
+                        'val': 1
+                    }
+                }
             }
         }
         assert expected == ddiff
@@ -1526,3 +1597,72 @@ class TestDeltaCompareFunc:
         delta = Delta(ddiff)
         recreated_t2 = t1 + delta
         assert t2 == recreated_t2
+
+    def test_compare_func_nested_changes(self):
+        t1 = {
+            "TestTable": [
+                {
+                    "id": "022fb580-800e-11ea-a361-39b3dada34b5",
+                    "name": "Max",
+                    "NestedTable": [
+                        {
+                            "id": "022fb580-800e-11ea-a361-39b3dada34a6",
+                            "NestedField": "Test Field"
+                        }
+                    ]
+                },
+                {
+                    "id": "022fb580-800e-11ea-a361-12354656532",
+                    "name": "Bob",
+                    "NestedTable": [
+                        {
+                            "id": "022fb580-800e-11ea-a361-39b3dada34c7",
+                            "NestedField": "Test Field 2"
+                        },
+                    ]
+                },
+            ]
+        }
+        t2 = {"TestTable": [
+            {
+                "id": "022fb580-800e-11ea-a361-12354656532",
+                "name": "Bob (Changed Name)",
+                "NestedTable": [
+                    {
+                        "id": "022fb580-800e-11ea-a361-39b3dada34c7",
+                        "NestedField": "Test Field 2 (Changed Nested Field)"
+                    },
+                    {
+                        "id": "new id",
+                        "NestedField": "Test Field 3"
+                    },
+                    {
+                        "id": "newer id",
+                        "NestedField": "Test Field 4"
+                    },
+                ]
+            },
+            {
+                "id": "adding_some_random_id",
+                "name": "New Name",
+                "NestedTable": [
+                    {
+                        "id": "random_nested_id_added",
+                        "NestedField": "New Nested Field"
+                    },
+                    {
+                        "id": "random_nested_id_added2",
+                        "NestedField": "New Nested Field2"
+                    },
+                    {
+                        "id": "random_nested_id_added3",
+                        "NestedField": "New Nested Field43"
+                    },
+                ]
+            }
+        ]}
+
+        ddiff = DeepDiff(t1, t2, iterable_compare_func=self.compare_func, verbose_level=2)
+        delta = Delta(ddiff)
+        recreated_t2 = t1 + delta
+        assert t2 == recreated_t2
