@@ -3,7 +3,7 @@ import datetime
 import pytest
 import logging
 import uuid
-import numpy as np
+from enum import Enum
 from decimal import Decimal
 from deepdiff import DeepDiff
 from deepdiff.helper import pypy3
@@ -82,20 +82,25 @@ class TestDeepDiffText:
         assert result == DeepDiff(t1, t2)
 
     def test_item_added_and_removed(self):
-        t1 = {1: 1, 2: 2, 3: 3, 4: 4}
-        t2 = {1: 1, 2: 4, 3: 3, 5: 5, 6: 6}
+        t1 = {1: 1, 2: 2, 3: [3], 4: 4}
+        t2 = {1: 1, 2: 4, 3: [3, 4], 5: 5, 6: 6}
         ddiff = DeepDiff(t1, t2)
         result = {
-            'dictionary_item_added': {'root[5]', 'root[6]'},
-            'dictionary_item_removed': {'root[4]'},
+            'dictionary_item_added': ["root[5]", "root[6]"],
+            'dictionary_item_removed': ["root[4]"],
             'values_changed': {
                 'root[2]': {
-                    "old_value": 2,
-                    "new_value": 4
+                    'new_value': 4,
+                    'old_value': 2
                 }
+            },
+            'iterable_item_added': {
+                'root[3][1]': 4
             }
         }
         assert result == ddiff
+        assert {'root[4]', 'root[5]', 'root[6]', 'root[3][1]', 'root[2]'} == ddiff.affected_paths
+        assert {4, 5, 6, 3, 2} == ddiff.affected_root_keys
 
     def test_item_added_and_removed_verbose(self):
         t1 = {1: 1, 3: 3, 4: 4}
@@ -421,44 +426,14 @@ class TestDeepDiffText:
         t1 = {1: 1, 2: 2, 3: 3, 4: {"a": "hello", "b": [1, 2, 5]}}
         t2 = {1: 1, 2: 2, 3: 3, 4: {"a": "hello", "b": [1, 3, 2, 5]}}
         ddiff = DeepDiff(t1, t2)
-        result = {
-            'values_changed': {
-                "root[4]['b'][2]": {
-                    'new_value': 2,
-                    'old_value': 5
-                },
-                "root[4]['b'][1]": {
-                    'new_value': 3,
-                    'old_value': 2
-                }
-            },
-            'iterable_item_added': {
-                "root[4]['b'][3]": 5
-            }
-        }
-        assert result == ddiff
+        expected = {'iterable_item_added': {"root[4]['b'][1]": 3}}
+        assert expected == ddiff
 
     def test_list_difference4(self):
-        # TODO: Look into Levenshtein algorithm
-        # So that the result is just insertion of "c" in this test.
         t1 = ["a", "b", "d", "e"]
         t2 = ["a", "b", "c", "d", "e"]
         ddiff = DeepDiff(t1, t2)
-        result = {
-            'values_changed': {
-                'root[2]': {
-                    'new_value': 'c',
-                    'old_value': 'd'
-                },
-                'root[3]': {
-                    'new_value': 'd',
-                    'old_value': 'e'
-                }
-            },
-            'iterable_item_added': {
-                'root[4]': 'e'
-            }
-        }
+        result = {'iterable_item_added': {'root[2]': 'c'}}
         assert result == ddiff
 
     def test_list_of_booleans(self):
@@ -550,7 +525,6 @@ class TestDeepDiffText:
         assert result == ddiff
 
     def test_enums(self):
-        from enum import Enum
 
         class MyEnum(Enum):
             A = 1
@@ -563,14 +537,6 @@ class TestDeepDiffText:
         ddiff = DeepDiff(MyEnum.A, MyEnum.B)
         result = {
             'values_changed': {
-                'root._name_': {
-                    'old_value': 'A',
-                    'new_value': 'B'
-                },
-                'root._value_': {
-                    'old_value': 1,
-                    'new_value': 2
-                },
                 'root.name': {
                     'old_value': 'A',
                     'new_value': 'B'
@@ -1310,6 +1276,15 @@ class TestDeepDiffText:
         ddiff = DeepDiff(t1, t2, exclude_paths={"root['ingredients']"})
         assert {} == ddiff
 
+    def test_skip_path2_key_names(self):
+        t1 = {
+            "for life": "vegan",
+            "ingredients": ["no meat", "no eggs", "no dairy"]
+        }
+        t2 = {"for life": "vegan"}
+        ddiff = DeepDiff(t1, t2, exclude_paths={"ingredients"})
+        assert {} == ddiff
+
     def test_skip_path2_reverse(self):
         t1 = {
             "for life": "vegan",
@@ -1319,6 +1294,43 @@ class TestDeepDiffText:
         ddiff = DeepDiff(t2, t1, exclude_paths={"root['ingredients']"})
         assert {} == ddiff
 
+    def test_include_path3(self):
+        t1 = {
+            "for life": "vegan",
+            "ingredients": ["no meat", "no eggs", "no dairy"]
+        }
+        t2 = {"for life": "vegan"}
+        ddiff = DeepDiff(t2, t1, include_paths={"root['for_life']"})
+        assert {} == ddiff
+
+    def test_include_path3_with_just_key_names(self):
+        t1 = {
+            "for life": "vegan",
+            "ingredients": ["no meat", "no eggs", "no dairy"]
+        }
+        t2 = {"for life": "vegan"}
+        ddiff = DeepDiff(t1, t2, include_paths={"for_life"})
+        assert {} == ddiff
+
+    def test_include_path4_nested(self):
+        t1 = {
+            "foo": {"bar": "potato"},
+            "ingredients": ["no meat", "no eggs", "no dairy"]
+        }
+        t2 = {
+            "foo": {"bar": "banana"},
+            "ingredients": ["bread", "cheese"]
+        }
+        ddiff = DeepDiff(t1, t2, include_paths="foo")
+        assert {
+            'values_changed': {
+                "root['foo']['bar']": {
+                    'new_value': 'banana',
+                    'old_value': 'potato'
+                }
+            }
+        } == ddiff
+
     def test_skip_path4(self):
         t1 = {
             "for life": "vegan",
@@ -1394,6 +1406,18 @@ class TestDeepDiffText:
         result = {}
         assert result == ddiff
 
+    def test_skip_exclude_obj_callback_strict(self):
+        def exclude_obj_callback_strict(obj, path):
+            return True if isinstance(obj, int) and obj > 10 else False
+
+        t1 = {"x": 10, "y": "b", "z": "c"}
+        t2 = {"x": 12, "y": "b", "z": "c"}
+        ddiff = DeepDiff(t1, t2, exclude_obj_callback_strict=exclude_obj_callback_strict)
+        result = {'values_changed': {"root['x']": {'new_value': 12, 'old_value': 10}}}
+        assert result == ddiff
+        assert {"root['x']"} == ddiff.affected_paths
+        assert {"x"} == ddiff.affected_root_keys
+
     def test_skip_str_type_in_dictionary(self):
         t1 = {1: {2: "a"}}
         t2 = {1: {}}
@@ -1446,6 +1470,36 @@ class TestDeepDiffText:
             'iterable_item_removed': {'root[2]': None}
         }
         assert result == ddiff
+        assert {"root[2]"} == ddiff.affected_paths
+
+    def test_list_item_removed_from_the_middle(self):
+        t1 = [0, 1, 2, 3, 'bye', 5, 6, 7, 8, 'a', 'b', 'c']
+        t2 = [0, 1, 2, 3, 5, 6, 7, 8, 'a', 'b', 'c']
+        diff = DeepDiff(t1, t2)
+        result = {'iterable_item_removed': {'root[4]': 'bye'}}
+        assert result == diff
+        assert {"root[4]"} == diff.affected_paths
+        assert {4} == diff.affected_root_keys
+
+    def test_list_item_values_replace_in_the_middle(self):
+        t1 = [0, 1, 2, 3, 'bye', 5, 6, 7, 8, 'a', 'b', 'c']
+        t2 = [0, 1, 2, 3, 'see', 'you', 'later', 5, 6, 7, 8, 'a', 'b', 'c']
+        diff = DeepDiff(t1, t2)
+        result = {
+            'values_changed': {
+                'root[4]': {
+                    'old_value': 'bye',
+                    'new_value': 'see',
+                }
+            },
+            'iterable_item_added': {
+                'root[5]': 'you',
+                'root[6]': 'later'
+            }
+        }
+        assert result == diff
+        assert {'root[5]', 'root[6]', 'root[4]'} == diff.affected_paths
+        assert {4, 5, 6} == diff.affected_root_keys
 
     def test_non_subscriptable_iterable(self):
         def gen1():
@@ -1465,6 +1519,7 @@ class TestDeepDiffText:
         # Note: In text-style results, we currently pretend this stuff is subscriptable for readability
 
         assert result == ddiff
+        assert {"root[2]"} == ddiff.affected_paths
 
     @pytest.mark.parametrize('t1, t2, params, expected_result', [
         (float('nan'), float('nan'), {}, ['values_changed']),
@@ -1593,6 +1648,7 @@ class TestDeepDiffText:
         diff = DeepDiff(t1, t2, group_by='id')
         expected = {'values_changed': {'root[1]': {'new_value': 3, 'old_value': 2}}}
         assert expected == diff
+        assert {"root[1]"} == diff.affected_paths
 
     def test_datetime_in_key(self):
 
@@ -1657,3 +1713,34 @@ class TestDeepDiffText:
         }
 
         assert expected2 == diff2
+
+    def test_diffs_rrules(self):
+
+        from dateutil.rrule import MONTHLY, rrule
+
+        d = DeepDiff(
+            rrule(freq=MONTHLY, count=5, dtstart=datetime.datetime(2014, 12, 31)),
+            rrule(freq=MONTHLY, count=4, dtstart=datetime.datetime(2011, 12, 31)),
+        )
+
+        assert d == {
+            "values_changed": {
+                "root[0]": {
+                    "new_value": datetime.datetime(2011, 12, 31, 0, 0),
+                    "old_value": datetime.datetime(2014, 12, 31, 0, 0),
+                },
+                "root[1]": {
+                    "new_value": datetime.datetime(2012, 1, 31, 0, 0),
+                    "old_value": datetime.datetime(2015, 1, 31, 0, 0),
+                },
+                "root[2]": {
+                    "new_value": datetime.datetime(2012, 3, 31, 0, 0),
+                    "old_value": datetime.datetime(2015, 3, 31, 0, 0),
+                },
+                "root[3]": {
+                    "new_value": datetime.datetime(2012, 5, 31, 0, 0),
+                    "old_value": datetime.datetime(2015, 5, 31, 0, 0),
+                },
+            },
+            "iterable_item_removed": {"root[4]": datetime.datetime(2015, 7, 31, 0, 0)},
+        }
