@@ -47,6 +47,7 @@ class TestIgnoreOrder:
         t2 = [[{4, 5, 6}], {1, 2, 3}]
         ddiff = DeepDiff(t1, t2, ignore_order=True)
         assert {'set_item_added': ["root[1][0][6]"]} == ddiff
+        assert {"root[1][0][6]"} == ddiff.affected_paths
 
     def test_ignore_order_depth4(self):
         t1 = [[1, 2, 3, 4], [4, 2, 2, 1]]
@@ -74,6 +75,7 @@ class TestIgnoreOrder:
             }
         }
         assert expected == ddiff
+        assert {"root[1]", "root[2]", "root[3]"} == ddiff.affected_paths
 
         ddiff = DeepDiff(t1, t2, ignore_order=True, report_repetition=False, cache_purge_level=0)
         dist = ddiff._get_rough_distance()
@@ -124,6 +126,7 @@ class TestIgnoreOrder:
         t2 = {"a": [[{"b": 2, "c": 3}, {"b": 2, "c": 4}]]}
         ddiff = DeepDiff(t1, t2, ignore_order=True)
         assert {} == ddiff
+        assert set() == ddiff.affected_paths
 
     def test_nested_list_ignore_order(self):
         t1 = [1, 2, [3, 4]]
@@ -190,6 +193,7 @@ class TestIgnoreOrder:
             }
         }
         assert result != ddiff
+        assert {"root[2][0]"} == ddiff.affected_paths
 
     def test_list_of_unhashable_difference_ignore_order(self):
         t1 = [{"a": 2}, {"b": [3, 4, {1: 1}]}]
@@ -749,7 +753,7 @@ class TestIgnoreOrder:
 
         assert expected == diff_with_dist
 
-    def test_ignore_order_and_group_by(self):
+    def test_ignore_order_and_group_by1(self):
         t1 = [
             {'id': 'AA', 'name': 'Joe', 'ate': ['Nothing']},
             {'id': 'BB', 'name': 'James', 'ate': ['Chips', 'Cheese']},
@@ -763,24 +767,116 @@ class TestIgnoreOrder:
         ]
 
         diff = DeepDiff(t1, t2, group_by='id', ignore_order=False)
-        expected = {
-            'values_changed': {
-                "root['BB']['ate'][1]": {
-                    'new_value': 'Brownies',
-                    'old_value': 'Cheese'
-                }
-            },
-            'iterable_item_added': {
-                "root['CC']['ate'][1]": 'Apple',
-                "root['BB']['ate'][2]": 'Cheese'
-            }
-        }
+        expected = {'iterable_item_added': {"root['BB']['ate'][1]": 'Brownies', "root['CC']['ate'][1]": 'Apple'}}
         assert expected == diff
 
         diff2 = DeepDiff(t1, t2, group_by='id', ignore_order=True)
         expected2 = {'iterable_item_added': {"root['BB']['ate'][1]": 'Brownies'}}
         assert expected2 == diff2
 
+    def test_ignore_order_and_group_by2(self):
+        t1_data = [{'id': '1', 'codes': ['1', '2', '3']}]
+        t2_data = [{'id': '1', 'codes': ['1', '2', '4']}]
+        diff = DeepDiff(t1_data, t2_data, group_by='id', ignore_order=True)
+        expected = {'values_changed': {"root['1']['codes'][2]": {'new_value': '4', 'old_value': '3'}}}
+        assert expected == diff
+
+    def test_ignore_order_and_group_by3(self):
+        t1 = [{
+            'id':
+            '5ec52e',
+            'products': [{
+                'lineNumber': 1,
+                'productPrice': '2.39',
+                'productQuantity': 2
+            }, {
+                'lineNumber': 2,
+                'productPrice': '4.44',
+                'productQuantity': 1
+            }],
+        }]
+
+        t2 = [{
+            'id':
+            '5ec52e',
+            'products': [
+                {
+                    'lineNumber': 2,
+                    'productPrice': '4.44',
+                    'productQuantity': 1
+                },
+                {
+                    'lineNumber': 1,
+                    'productPrice': '2.39',
+                    'productQuantity': 2
+                },
+            ],
+        }]
+
+        diff = DeepDiff(t1, t2, group_by='id', ignore_order=True)
+        assert {} == diff
+
+    def test_ignore_order_and_group_by4(self):
+        t1 = [
+            {
+                "id": "1",
+                "field_01": {
+                    "subfield_01": {
+                        "subfield_02": {"subfield_03": "1"},
+                    }
+                },
+            },
+            {"id": "2", "field_01": ["1", "2", "3"]},
+            {"id": "3", "field_01": ["1", "2", "3"]},
+        ]
+        t2 = [
+            {
+                "id": "1",
+                "field_01": {
+                    "subfield_01": {
+                        "subfield_02": {"subfield_03": "2"},
+                    }
+                },
+            },
+            {"id": "2", "field_01": ["4", "5", "6"]},
+            {"id": "3", "field_01": ["7", "8", "9"]},
+        ]
+        diff = DeepDiff(t1, t2, group_by='id', ignore_order=True)
+        expected = {
+            'values_changed': {
+                "root['1']['field_01']['subfield_01']['subfield_02']['subfield_03']": {
+                    'new_value': '2',
+                    'old_value': '1'
+                },
+                "root['2']['field_01'][1]": {
+                    'new_value': '5',
+                    'old_value': '2'
+                },
+                "root['3']['field_01'][2]": {
+                    'new_value': '9',
+                    'old_value': '3'
+                },
+                "root['2']['field_01'][0]": {
+                    'new_value': '4',
+                    'old_value': '1'
+                },
+                "root['3']['field_01'][1]": {
+                    'new_value': '8',
+                    'old_value': '2'
+                },
+                "root['3']['field_01'][0]": {
+                    'new_value': '7',
+                    'old_value': '1'
+                },
+                "root['2']['field_01'][2]": {
+                    'new_value': '6',
+                    'old_value': '3'
+                }
+            }
+        }
+
+        assert expected == diff
+
 
 class TestCompareFuncIgnoreOrder:
 
@@ -1023,7 +1119,7 @@ class TestDecodingErrorIgnoreOrder:
         "Please either pass ignore_encoding_errors=True or pass the encoding via encodings=['utf-8', '...'].")
 
     EXPECTED_MESSAGE2 = (
-        "'utf-8' codec can't decode byte 0xbc in position 0: Can not produce a hash for root: invalid start byte in 'p of flo...'. "
+        "'utf-8' codec can't decode byte 0xbc in position 0: Can not produce a hash for root: invalid start byte in ' cup of flour'. "
         "Please either pass ignore_encoding_errors=True or pass the encoding via encodings=['utf-8', '...'].")
 
     @pytest.mark.parametrize('test_num, item, encodings, ignore_encoding_errors, expected_result, expected_message', [
